/*
 * Please do not edit this file.
 * It was generated using rpcgen.
 */

#ifndef _TESTDATA_H_RPCGEN
#define _TESTDATA_H_RPCGEN

#include <vmxrpc.h>


#ifdef __cplusplus
extern "C" {
#endif


struct TestPluginData {
	char *data;
	uint32 f_int;
	Bool f_bool;
};
typedef struct TestPluginData TestPluginData;

/* the xdr functions */

#if defined(__STDC__) || defined(__cplusplus)
extern  bool_t xdr_TestPluginData (XDR *, TestPluginData*);

#else /* K&R C */
extern bool_t xdr_TestPluginData ();

#endif /* K&R C */

#ifdef __cplusplus
}
#endif

#endif /* !_TESTDATA_H_RPCGEN */
