/*
 * Please do not edit this file.
 * It was generated using rpcgen.
 */

#include "guestlibIoctl.h"

bool_t
xdr_GuestLibAtomicUpdateStatus (XDR *xdrs, GuestLibAtomicUpdateStatus *objp)
{

	 if (!xdr_enum (xdrs, (enum_t *) objp))
		 return FALSE;
	return TRUE;
}

bool_t
xdr_GuestLibIoctlId (XDR *xdrs, GuestLibIoctlId *objp)
{

	 if (!xdr_enum (xdrs, (enum_t *) objp))
		 return FALSE;
	return TRUE;
}

bool_t
xdr_GuestLibIoctlAtomicUpdateCookie (XDR *xdrs, GuestLibIoctlAtomicUpdateCookie *objp)
{

	 if (!xdr_string (xdrs, &objp->src, GUESTLIB_IOCTL_ATOMIC_UPDATE_COOKIE_LENGTH))
		 return FALSE;
	 if (!xdr_string (xdrs, &objp->dst, GUESTLIB_IOCTL_ATOMIC_UPDATE_COOKIE_LENGTH))
		 return FALSE;
	return TRUE;
}

bool_t
xdr_GuestLibIoctlParam (XDR *xdrs, GuestLibIoctlParam *objp)
{

	 if (!xdr_GuestLibIoctlId (xdrs, &objp->d))
		 return FALSE;
	switch (objp->d) {
	case GUESTLIB_IOCTL_ATOMIC_UPDATE_COOKIE:
		 if (!xdr_GuestLibIoctlAtomicUpdateCookie (xdrs, &objp->GuestLibIoctlParam_u.atomicUpdateCookie))
			 return FALSE;
		break;
	default:
		return FALSE;
	}
	return TRUE;
}
