/*
 * Please do not edit this file.
 * It was generated using rpcgen.
 */

#include "testData.h"

bool_t
xdr_TestPluginData (XDR *xdrs, TestPluginData *objp)
{

	 if (!xdr_string (xdrs, &objp->data, 128))
		 return FALSE;
	 if (!xdr_uint32 (xdrs, &objp->f_int))
		 return FALSE;
	 if (!xdr_Bool (xdrs, &objp->f_bool))
		 return FALSE;
	return TRUE;
}
