/* This file is generated by glib-genmarshal, do not modify it. This code is licensed under the same license as the containing project. Note that it links to GLib, so must comply with the LGPL linking clauses. */
#ifndef __G_CCLOSURE_USER_MARSHAL_MARSHAL_H__
#define __G_CCLOSURE_USER_MARSHAL_MARSHAL_H__

#include <glib-object.h>

G_BEGIN_DECLS

/* POINTER:POINTER,BOOLEAN (../../services/vmtoolsd/svcSignals.gm:28) */
extern
void g_cclosure_user_marshal_POINTER__POINTER_BOOLEAN (GClosure     *closure,
                                                       GValue       *return_value,
                                                       guint         n_param_values,
                                                       const GValue *param_values,
                                                       gpointer      invocation_hint,
                                                       gpointer      marshal_data);

/* BOOLEAN:POINTER,STRING,STRING (../../services/vmtoolsd/svcSignals.gm:31) */
extern
void g_cclosure_user_marshal_BOOLEAN__POINTER_STRING_STRING (GClosure     *closure,
                                                             GValue       *return_value,
                                                             guint         n_param_values,
                                                             const GValue *param_values,
                                                             gpointer      invocation_hint,
                                                             gpointer      marshal_data);

/* UINT:POINTER,POINTER,UINT,UINT,POINTER (../../services/vmtoolsd/svcSignals.gm:35) */
extern
void g_cclosure_user_marshal_UINT__POINTER_POINTER_UINT_UINT_POINTER (GClosure     *closure,
                                                                      GValue       *return_value,
                                                                      guint         n_param_values,
                                                                      const GValue *param_values,
                                                                      gpointer      invocation_hint,
                                                                      gpointer      marshal_data);


G_END_DECLS

#endif /* __G_CCLOSURE_USER_MARSHAL_MARSHAL_H__ */
