/*
 * Please do not edit this file.
 * It was generated using rpcgen.
 */

#ifndef _CPFILECONTENTS_H_RPCGEN
#define _CPFILECONTENTS_H_RPCGEN

#include <vmxrpc.h>


#ifdef __cplusplus
extern "C" {
#endif


enum CPFileContentsVersion {
	CP_FILE_CONTENTS_V1 = 1,
};
typedef enum CPFileContentsVersion CPFileContentsVersion;

struct CPFileItem {
	uint64 validFlags;
	uint32 type;
	uint64 size;
	uint64 createTime;
	uint64 accessTime;
	uint64 writeTime;
	uint64 attrChangeTime;
	uint64 permissions;
	uint64 attributes;
	struct {
		u_int cpName_len;
		char *cpName_val;
	} cpName;
	struct {
		u_int content_len;
		char *content_val;
	} content;
};
typedef struct CPFileItem CPFileItem;

struct CPFileContentsList {
	uint64 totalFileSize;
	struct {
		u_int fileItem_len;
		struct CPFileItem *fileItem_val;
	} fileItem;
};
typedef struct CPFileContentsList CPFileContentsList;

struct CPFileContents {
	CPFileContentsVersion ver;
	union {
		struct CPFileContentsList *fileContentsV1;
	} CPFileContents_u;
};
typedef struct CPFileContents CPFileContents;

/* the xdr functions */

#if defined(__STDC__) || defined(__cplusplus)
extern  bool_t xdr_CPFileContentsVersion (XDR *, CPFileContentsVersion*);
extern  bool_t xdr_CPFileItem (XDR *, CPFileItem*);
extern  bool_t xdr_CPFileContentsList (XDR *, CPFileContentsList*);
extern  bool_t xdr_CPFileContents (XDR *, CPFileContents*);

#else /* K&R C */
extern bool_t xdr_CPFileContentsVersion ();
extern bool_t xdr_CPFileItem ();
extern bool_t xdr_CPFileContentsList ();
extern bool_t xdr_CPFileContents ();

#endif /* K&R C */

#ifdef __cplusplus
}
#endif

#endif /* !_CPFILECONTENTS_H_RPCGEN */
